#!/usr/bin/env python
# -*- coding: utf-8 -*-

############################################################
#
# Copyright (C) 2019 SenseDeal AI, Inc. All Rights Reserved
#
############################################################

"""
File: server.py
Author: liuguangbin
Editor: liuguangbin
E-mail:
Last modified: 2019-08-12 16:15
Description:
"""

import os
import re
import sys
import grpc
import json
import time

from rpc import token_ner_pb2, token_ner_pb2_grpc

from concurrent import futures
from configobj import ConfigObj

from lib.InferenceSolverOfNews import InferenceSolver
from lib.news_token_ner import TokenNER
from sense_core import config, log_init_config
from sense_file.load_file import log_alg_exec

log_init_config('token_ner_new', config('log_path'))
os.environ["CUDA_VISIBLE_DEVICES"] = ""


@log_alg_exec
def serve(mode):
    _confobj = ConfigObj('/conf/all.ini', encoding='utf8')
    _conf = _confobj[mode]
    _host = _conf['Alg']['news_token_ner']['node_host']
    _port = _conf['Alg']['news_token_ner']['node_port']
    alg_version = _conf['Alg']['news_token_ner']['version']

    _model_base = _conf['Alg']['news_token_ner']['modelpath']
    _embedding_path = _model_base

    ckpt_dirs = []
    data_dirs = []

    for file_name in os.listdir(_model_base):
        model_path = os.path.join(_model_base, file_name)
        if os.path.isdir(model_path) and 'seg' not in model_path:
            ckpt_dirs.append(os.path.join(model_path, 'checkpoints'))
            data_dirs.append(os.path.join(model_path, 'data'))

    _workers = int(_conf['Alg']['news_token_ner']['workers'])
    _solver = InferenceSolver(
        os.path.join(_model_base, 'seg_data/'),
        os.path.join(_model_base, 'seg_checkpoint/'),
        data_dirs,
        ckpt_dirs,
        _embedding_path
    )
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=_workers))
    token_ner_pb2_grpc.add_TokenNERServicer_to_server(TokenNER(_solver, alg_version), server)
    server.add_insecure_port(_host + ':' + _port)
    server.start()
    _ONE_DAY_IN_SECONDS = 60 * 60 * 24
    print(_host,_port , _model_base)
    while True:
        time.sleep(_ONE_DAY_IN_SECONDS)

if __name__ == '__main__':
    mode = str(sys.argv[1])
    serve(mode)


import configparser
from sense_core import config, log_warn
from sense_file import S3File
import tarfile


def log_alg_exec(function):
    def inner(mode):
        if mode == 'TestEnv':
            return function(mode)
        s3_file = S3File('aws')
        # 加载conf配置
        aim_conf_path = config('conf_path')
        conf_file = config('s3', 'config')
        conf_bucket = config('s3', 'conf_bucket')
        _code, _msg = s3_file.download(conf_file, aim_conf_path, conf_bucket)
        try:
            # 将s3 bucket中的s3_model_tar下载到本地model_tar位置，并解压到model_path下
            s3_model_tar = config('s3', 'model_tar')
            s3_bucket = config('s3', 'model_bucket')
            model_tar = config('model_tar')
            model_path = config('model_path')
        except configparser.NoOptionError:
            log_warn('no model load')
            return None
        _code, _msg = s3_file.download(s3_model_tar, model_tar, s3_bucket)
        if not _code:
            return None
        t = tarfile.open(model_tar)
        t.extractall(path=model_path)
        return function(mode)
    return inner


@log_alg_exec
def serve(mode):
    print('end')


if __name__ == '__main__':
    serve('TestEnv')
    # 加载conf配置
    # _code, _msg = s3_file.download('alg_access_config/all.ini', './all1.ini', 'sdai-conf')
    # s3_model_tar = config('s3_model_tar')


import datetime
from sense_core import get_db_factory, DBACTION


def upload_db_logs(function):
    def inner(self, file_name, origin_path, bucket_name):
        _success, _msg = function(self, file_name, origin_path, bucket_name)
        if _success:
            key = file_name.split('/')[-1]
            db_factory = get_db_factory('mysql_file')
            _now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            _params = {'key': key, 'bucket_name': bucket_name,
                       'create_time': _now, 'modify_time': _now,
                       'status': 1}
            db_factory.findKeySql(DBACTION.REPLACE, table='file_sheet', data=_params)
        return _success, _msg
    return inner


def upload_pdf_logs(function):
    def inner(self, file_name, origin_path, bucket_name, publish_time=None, url=None, stock_code=None):
        _success, _msg = function(self, file_name, origin_path, bucket_name, publish_time, url, stock_code)
        if _success:
            key = file_name.split('/')[-1]
            db_factory = get_db_factory('mysql_file')
            _now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            _params = {'key': key, 'bucket_name': bucket_name,
                       'create_time': _now, 'modify_time': _now,
                       'status': 1, 'publish_time': publish_time,
                       'publish_day': publish_time, 'stock_code': stock_code, 'url': url}
            db_factory.findKeySql(DBACTION.REPLACE, table='pdfs_sheet', data=_params)
        return _success, _msg
    return inner


def upload_model_logs(function):
    def inner(self, file_name, origin_path, bucket_name, alg):
        _success, _msg = function(self, file_name, origin_path, bucket_name, alg)
        if _success:
            key = file_name.split('/')[-1]
            db_factory = get_db_factory('mysql_file')
            _now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            _params = {'key': key, 'bucket_name': bucket_name,
                       'create_time': _now, 'modify_time': _now,
                       'status': 1, 'alg': alg}
            db_factory.findKeySql(DBACTION.REPLACE, table='model_sheet', data=_params)
        return _success, _msg
    return inner


def delete_db_logs(function):
    def inner(self, file_name, bucket_name):
        _success, key = function(self, file_name, bucket_name)
        if _success:
            db_factory = get_db_factory('mysql_file')
            _now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            db_factory.findKeySql(DBACTION.DELETE_BY_ATTR, table='file_sheet', params={'key': key})
        return _success, key

    return inner



import os
from setuptools import setup

requirements = [
    "boto3",
    "sense-core>=0.2.39"
]

# with open("README.md", "r", encoding='utf-8') as fh:
#     long_description = fh.read()

setup(
    name='sense-file',
    version='0.0.6',
    packages=[
            "sense_file",
    ],
    license='BSD License',  # example license
    description='sense file',
    install_requires=requirements,
    # long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://upload.pypi.org/legacy/',
    author='liuguangbin',
    author_email='liuguangbin@sensedeal.ai',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
    ],
)


import os
from boto3.session import Session
from sense_core import config, log_error
from sense_file.decorate import delete_db_logs, upload_db_logs, upload_pdf_logs, upload_model_logs


class S3File(object):

    def __init__(self, label, region_name='cn-north-1'):
        _key, _secret = os.getenv('S3_KEY', None), os.getenv('S3_SECRET', None)
        if not _key:
            _key, _secret = config(label, 'key'), config(label, 'secret')
        session = Session(aws_access_key_id=_key, aws_secret_access_key=_secret,
                          region_name=region_name)
        self.session = session
        self.server = session.resource('s3')
        self.client = None
        self.bucket_map = {}

    def __get_client(self):
        if not self.client:
            self.client = self.session.client('s3')
        return self.client

    def __file_key_format(self, file_name):
        file_list = file_name.split('.')
        file_type = file_list[-1]
        file_list[-1] = file_type.lower()
        return '.'.join(file_list)

    def __get_bucket(self, bucket_name):
        if bucket_name not in self.bucket_map:
            self.bucket_map[bucket_name] = self.server.Bucket(bucket_name)
        return self.bucket_map[bucket_name]

    def file_exists(self, file_name, bucket_name):
        bucket = self.__get_bucket(bucket_name)
        for obj in bucket.objects.filter(Prefix=file_name):
            if obj.key == file_name:
                return True
        return False

    def download(self, file_name, aim_path, bucket_name):
        """
        下载文件接口
        :param file_name:
        :param aim_path:
        :param bucket_name:
        :return:
        """
        file_name = self.__file_key_format(file_name)
        if not self.file_exists(file_name, bucket_name):
            return False, 'file not exists'
        try:
            self.server.Object(bucket_name, file_name).download_file(aim_path)
        except Exception as e:
            log_error(
                'file {} down from s3 {} error,msg: {}'.format(file_name, bucket_name, e.__str__()))
            return False, 'down error {}'.format(e.__str__())
        return os.path.exists(aim_path), file_name

    def __s3_upload_file(self, bucket_name, file_name, origin_path):
        _msg = '{}-{} upload file success '.format(file_name, bucket_name)
        try:
            obj = self.server.Object(bucket_name, file_name)
            obj.upload_file(origin_path)
        except Exception as ex:
            _msg = 'file {} upload from s3 {} error,msg: {}'.format(file_name, bucket_name,
                                                                    ex.__str__())
            log_error(_msg)
            return False, _msg
        return True, _msg

    @upload_pdf_logs
    def upload_pdf(self, file_name, origin_path, bucket_name, publish_time=None, url=None, stock_code=None):
        """ 上传公告，并写日志
        :param file_name: 文件名称
        :param origin_path: 源路径（本地路径）
        :param bucket_name:  aws s3桶
        :param publish_time,stock_code,url 装饰器使用字段来记录日志中
        :return:
        """
        file_name = self.__file_key_format(file_name)
        code, msg = self.__s3_upload_file(bucket_name, file_name, origin_path)
        return self.file_exists(file_name, bucket_name), msg

    @upload_model_logs
    def upload_model(self, file_name, origin_path, bucket_name, alg):
        """ 上传模型文件或配置，并写日志
        :param file_name: 文件名称
        :param origin_path: 源路径（本地路径）
        :param bucket_name:  aws s3桶
        :return:
        """
        code, msg = self.__s3_upload_file(bucket_name, file_name, origin_path)
        return self.file_exists(file_name, bucket_name), msg

    @upload_db_logs
    def upload(self, file_name, origin_path, bucket_name):
        """ 上传文件
        :param file_name: 文件名称
        :param origin_path: 源路径（本地路径）
        :param bucket_name:  aws s3桶
        :return:
        """
        file_name = self.__file_key_format(file_name)
        code, msg = self.__s3_upload_file(bucket_name, file_name, origin_path)
        return self.file_exists(file_name, bucket_name), msg

    @delete_db_logs
    def delete_file(self, file_name, bucket_name):
        file_name = self.__file_key_format(file_name)
        try:
            file_obj = self.server.Object(bucket_name, file_name)
            res = file_obj.delete()
        except Exception as e:
            log_error('file {} delete from s3 {} error,msg: {}'.format(file_name, bucket_name,
                                                                       e.__str__()))
            return False, 'delete error {}'.format(e.__str__())
        return True, file_name

    @upload_db_logs
    def upload_content(self, file_name, content, bucket_name, publish_time=None):
        file_name = self.__file_key_format(file_name)
        try:
            client = self.__get_client()
            client.put_object(Bucket=bucket_name, Key=file_name,
                              Body=bytes(content, encoding='utf-8'))
        except Exception as e:
            log_error('file {} upload from s3 {} error,msg: {}'.format(file_name, bucket_name,
                                                                       e.__str__()))
            return False, 'upload content error {}'.format(e.__str__())
        return self.file_exists(file_name, bucket_name), file_name


if __name__ == '__main__':
    s3_file = S3File('aws')
    _path = '/Users/liuguangbin/Documents/alg/conf/news_post_v2019081215.ini'
    # print(s3_file.download('tttt.txt', '/Users/liuguangbin/Documents/ttttt.txt',
    #                        'sdai-txt'))
    # print(s3_file.delete_file('tttt.txt', 'sdai-txt'))
    # x = s3_file.file_exists('000b9ccb-76d7-5421-997d-e3b8413479a2', 'sdai-pdfs')
    # print(s3_file.upload('000b9ccb-76d7-5421-997d-e3b8413479a2.PDF', file_path, 'sdai-pdfs'))
